/* XND_REXP.C   read the experimental parameters */
static char *cvs_id =" $Id: xnd_rexp.c,v 1.4 2002/06/19 09:57:29 berar Exp $ ";

#define USE_VAR
#define USE_KEYS
#define USE_SCAN
#define USE_OBS
#define USE_EXPER
#include "../xnd_def.h"
#include "xnd_rdh.h"
#include "../xnd_mem.h"

#define EPS_CHK 0.01

/* ------------------------------------------------------------------ */
static char *txt_onde[LAMBDA]    = {"Lamda","Int.","WlT-t","WgT-t"};
static char *txt_settings_tra[]  = {"Zero","Polar","Tmono","S_Ecl","Alpha","Abs_cyl","Abs_pl"};
static char *txt_settings_ref[]  = {"Zero","Polar","Tmono","S_Ecl","Alpha","Alpha_c","Mu"};
static char *txt_instru[]        = {"Wl0","Wg0","Wl_T2","Wg_T2","WlSa","WgSa","WlS2a","WgS2a"};
static char *txt_asym[ASYM]      = {"A_T2","A_T","AS2"};
static char *txt_ascnv[ASCNV]    = {"H+S_L","D_H+S"};
static char *txt_c_exp[1]        = {"Resol"};
/* ------------------------------------------------------------------ */
void write_chk (P_(void))
{    TYPE_OBS obs;
     obs.dtheta = CHK_VAL;
     obs.i_obs  = l_diag->temp;
     obs.weight = l_diag->w_mini;
     data_write (&obs);
     obs.i_obs  = l_diag->dtheta0;
     obs.weight = l_diag->dthetaf;
     data_write (&obs);
     }
/* ------------------------------------------------------------------ */
FLOAT read_chk(P_(void))
{    TYPE_OBS obs;
     FLOAT t;
     data_read(&obs);
     tst_abort ((fabs((double)obs.dtheta-CHK_VAL)>EPS_CHK),"Verif data_fich");
     t = obs.i_obs;
     l_diag->w_mini = obs.weight;
     data_read(&obs);
     tst_abort ((fabs(obs.dtheta-CHK_VAL)>EPS_CHK),"Verif data_fich");
     l_diag->dtheta0 = obs.i_obs;
     l_diag->dthetaf = obs.weight;
     return t;
     }
/* ------------------------------------------------------------------ */
/* ------------------------------------------------------------------ */
static void pseudo_data (P_(void))
{    int    i, nb, num;
     TYPE_OBS    obs;
	  float  step;
     
	 push_err_msg("pseudo_data(xnd_rexp.c)");
	 for (num=0; num < l_exper->n_diag; num++,l_diag++)
    {    l_diag->w_mini = 1;
         purge_cr();
			l_diag->dtheta0 = KEY_DATA_START;
			l_diag->dthetaf = KEY_DATA_STOP;
			step = KEY_DATA_STEP;
	if ( get_keys("DATA", NULL,
       "START",     KEYTYPE_FLOAT,    &(l_diag->dtheta0), &l_diag->dtheta0,
       "STOP",       KEYTYPE_FLOAT,    &(l_diag->dthetaf), &l_diag->dthetaf, 
       "STEP",      KEYTYPE_FLOAT,    &(step),            &step,
       "TEMP",      KEYTYPE_FLOAT,    &(l_diag->temp),    &l_diag->temp,
        NULL ) == -1 )
   /* the following allows xnd_1.17 header to be read */
	fscanf (coord_fich,"%f%f%f%f",&l_diag->temp,&l_diag->dtheta0,&step,&l_diag->dthetaf);
	fprintf (list_fich," PSEUDO_DIAG : T = %.3f, start=%.4f step=%.4f stop=%.4f\n",
		 l_diag->temp,l_diag->dtheta0, step, l_diag->dthetaf);
	tst_abort ((step <= MINFLOAT),"step lower or egal to zero");
	nb = 1 + floor ((l_diag->dthetaf-l_diag->dtheta0)/step);
	tst_abort ((nb <= 1),"no points between dtheta0 and dthetaf with such step");
        obs.dtheta = l_diag->dtheta0;
        obs.weight = obs.i_obs = 1;
        for (i=0;i<nb;i++,obs.dtheta += step) data_write (&obs);
        write_chk();
        l_diag->n_points = nb;
     lect->tot_points += l_diag->n_points;
     if (l_diag->n_points > lect->max_points)
		 lect->max_points = l_diag->n_points;
	  }
	pop_err_msg();
	}


/* ------------------------------------------------------------------ */
/* ------------------------------------------------------------------ */
int read_exper (P_(void))
{   char   *txt_nature, *txt_techn, *txt_fich, *txt_powder;
    char   *pluriel;
    int    i, nature, n_tech, n_arg, n_key;

    push_err_msg("read_exper(xnd_rexp.c)");
    memset(l_exper, 0, sizeof(TYPE_EXPER));
    l_exper->remark = read_remark(NULL);
    l_exper->weight = KEY_EXPER_WEIGHT;
    l_exper->temp_orig = KEY_EXPER_ORIG;

    n_key = get_keys("EXPER", NULL,
       "TITLE",      KEYTYPE_STRING,  l_exper->nom,       "",
       "NATURE",     KEYTYPE_INT,    &nature,             (int) KEY_EXPER_NATURE,
       "TECHN",      KEYTYPE_INT,    &(l_exper->techn),   (int) KEY_EXPER_TECHN,
       "FILE",       KEYTYPE_INT,    &(l_exper->dfich),   (int) KEY_EXPER_FILE,
       "DATA",       KEYTYPE_INT,    &(l_exper->n_diag),  (int) KEY_EXPER_DATA,
       "LAMBDA",     KEYTYPE_INT,    &(l_exper->n_lambda),(int) KEY_EXPER_LAMBDA,
       "BGROUND",    KEYTYPE_INT,    &(l_exper->n_back),  (int) KEY_EXPER_BGROUND,
       "ORIG",       KEYTYPE_FLOAT,  &(l_exper->temp_orig), &l_exper->temp_orig,
       "WEIGHT",     KEYTYPE_FLOAT,  &(l_exper->weight),    &l_exper->weight, 
        NULL );
   /* the following allows xnd_1.17 header to be read */
	if ( n_key == -1 )
	{  n_arg = fscanf (coord_fich," %d%d%d%d%d%d%f",&nature,
	    &l_exper->techn,&l_exper->n_diag,&l_exper->dfich,
	    &l_exper->n_lambda, &l_exper->n_back,
	    &l_exper->temp_orig);
       purge_cr();
       if ( n_arg == 7 ) n_arg += fscanf (coord_fich," %[^@\n]", l_exper->nom);
       /* end of xnd_1.17 header */
		 } else n_arg = 0; 
   purge_cr();

    switch (nature % 8)
	{	case 0  :  l_exper->nature = EXPER_RX;
			   txt_nature = "XRay"; break;
		case 1  :  l_exper->nature = EXPER_N;
			   txt_nature = "NEUTRONS"; break;
		case 2  :  l_exper->nature = EXPER_RXA;
			   txt_nature = "RX_ANOMAL"; break;
		case 3  :  l_exper->nature = EXPER_NM;
			   txt_nature = "NEUTRONS MAGNETIQUES"; break;
		}
/*	if ( nature & 16 )
	{   txt_ener = "Energy resolution" ;
		 l_exper->nature |= EXPER_ENER_RESOL;
		 }
	else txt_ener = "";      */
	if ( nature & 32 )
	{   txt_powder = "single crystal" ;
		 l_exper->nature |= EXPER_SINGLE_CRYSTAL;
		 }
	else txt_powder = "powder";
	n_tech = 4;
	switch (l_exper->techn )
	{	case 0 :  txt_techn = "Reflexion Sym";
			            break;
		case 1 :  txt_techn = "Reflexion Assym";
			  n_tech++; break;
		case 2 :  txt_techn = "Reflexion fixed incidence angle, linear detector ???";
			  n_tech++; break;
		case 3 :  txt_techn = "Reflexion fixed incidence angle, curved detector";
			  n_tech++; break;
	 	case 4 :  txt_techn = "Transmission Sym";
			  n_tech+=3; break;
		case 5 :  txt_techn = "Transmission Assym ???";
			  n_tech+=3; break;
		case 6 :  txt_techn = "Transmission fixed sample, linear detector ???";
			  n_tech+=3; break;
		case 7 :  txt_techn = "Transmission fixed sample, curved detector ???";
			  n_tech+=3; break;
		case 8 :  txt_techn = "Reflexion (absorption corrected)";
			  n_tech+=3; break;
		case 9 :  txt_techn = "Reflexion (absorption corrected)";
			  n_tech+=3; break;
		case 10 :  txt_techn = "Reflexion (absorption corrected) with fixed incidence angle, linear";
			  n_tech+=3; break;
		case 11 :  txt_techn = "Reflexion (absorption corrected) with fixed incidence angle, curved";
			  n_tech+=3; break;
		}
 	switch (l_exper->dfich)
	{	case 0 :  txt_fich = "SIMULATED PATTERN"; break;
		case 1 : txt_fich = " 2th i"; break;
		case 2 : txt_fich = " th 2th i"; break;
		case 3 : txt_fich = " 2th i s"; break;
		case 4 : txt_fich = " th 2th i s"; break;
		case 10 :  txt_fich = "exRAW"; break;
		case 11 :  txt_fich = "DBW"; break;
		case 12 : case 13 :
		case 14 : txt_fich = "LLB"; break;
		case 20 : txt_fich = "ILL_RT" ; break;
		case 21 : txt_fich = "VDMI_RT" ; break;
		case 50 : txt_fich = "hkl Fo So"; break;
		case 51 : txt_fich = "hkl Io So"; break;
		default : txt_fich = "???";
		}
	if (l_exper->n_diag > 1) pluriel = "s"; else pluriel = "";
	nt_diag += l_exper->n_diag;
   l_exper->m_back=1;
   if ( l_exper->n_back < 0 ) 
   {   l_exper->n_back = -l_exper->n_back;
       if (l_exper->n_diag > 1) l_exper->m_back = l_exper->n_diag;
       }
        
	fprintf(list_fich,"\n EXPER : %d (%s,%s), techn %d (%s)\n %d data set%s : file%s type %d (%s) \n",
		 nature, txt_nature, txt_powder,
		 l_exper->techn, txt_techn, l_exper->n_diag,
		 pluriel, pluriel, l_exper->dfich, txt_fich);
	if ( l_exper->nature & EXPER_SINGLE_CRYSTAL)
	fprintf(list_fich," phase_ref : %d, hkl_dim : %d\n",
		 l_exper->n_lambda, l_exper->n_back);
	else fprintf(list_fich," wavelength : %d, background : %d (* %d points): %s\n",
		 l_exper->n_lambda, l_exper->n_back, l_exper->m_back, 
		 l_exper->m_back == 1 ? "common to diags" : "for each diag");
	fprintf(list_fich," T_orig in expansion %.3f\n '%s'\n",
		 l_exper->temp_orig,	l_exper->nom);
	 if (bavard) printf("\n EXPER : '%s' : %d data set\n",
		 l_exper->nom, l_exper->n_diag);
	tst_abort( (n_key==-1 && n_arg!=8), "error reading experiment header keys=%d && args=%d!=8", n_arg, n_key);      
	tst_abort ((nature<0 || (nature%8)>3),"unknown experimental nature");
        tst_abort ((l_exper->techn<0 || l_exper->techn>11),"unknown experimental setup");
	tst_abort ((l_exper->n_diag==0),"Experiment without data set");
    
    l_diag = l_exper->diag = (TYPE_SCAN *) tst_alloc(l_exper->n_diag,sizeof(TYPE_SCAN),"patterns");
    switch  (l_exper->dfich)
    {   case 0  : pseudo_data ();   break;
	  case 20 :
	  case 21 : get_mult_data (); break;
	  case 50 : get_hkl_data (); break;
	  default : get_real_data ();
	  }

   fprintf(list_fich,"\n");
   l_exper->n_var = n_var;
	if (l_exper->dfich != 50)
   {  l_exper->onde=fget_var(LAMBDA*l_exper->n_lambda,LAMBDA,VAR_STD,
	       T_LAMBDA,(void *) txt_onde, NULL, NULL,
			 VAR_MODE_GT0 || VAR_MODE_ABS, 0.01 );
      l_exper->prof_par = 0;
      var_init(l_exper->onde);
      VAR_io_id
      for (i=0;i<l_exper->n_lambda;i++)
      {  if (var_param(i*LAMBDA+2)) l_exper->prof_par++;
	      if (var_param(i*LAMBDA+3)) l_exper->prof_par++;
	      }
      if ( back_mode == 2 || back_mode == 3)
         l_exper->back=fget_var(l_exper->n_back*l_exper->m_back, l_exper->n_back, VAR_POLY, 
			    T_BACK, NULL, NULL, NULL, 0 ,0);
      else
         l_exper->back=fget_var(l_exper->n_back*l_exper->m_back, l_exper->n_back, VAR_FLOAT, 
			T_BACK, NULL, NULL, NULL, 0 ,0);
      if ( l_exper->techn <= 7)
      l_exper->reglage=fget_var (n_tech, 0, VAR_STD, "SETTING",(void *) txt_settings_tra,
		    NULL, NULL, 0 ,0);
      else
      l_exper->reglage=fget_var (n_tech, 0, VAR_STD, "SETTING",(void *) txt_settings_ref,
		    NULL, NULL, 0 ,0);
      if ((l_exper->techn % 4) != 3 )
		   l_exper->instru=fget_var(INSTRU, 0, VAR_STD,"INSTRU",(void *) txt_instru, 
			    NULL, NULL, VAR_MODE_GT0 || VAR_MODE_ABS, 0.01);
      else
		   l_exper->instru=fget_var(INSTRU+4, 0, VAR_STD,"INSTRU",(void *) txt_instru, 
			    NULL, NULL, VAR_MODE_GT0 || VAR_MODE_ABS, 0.01);
      if ( asym_mode != ASYM_NO_CONVOL )
         l_exper->ascnv=fget_var(ASCNV, 0, VAR_STD, T_ASCNV, (void *) txt_ascnv, 
			    NULL, NULL, 0, 0);
      if ( max_asym > 0 )
         l_exper->asym=fget_var(max_asym*ASYM, ASYM, VAR_ORIEN, T_ASYM, (void *) txt_asym, 
			  NULL, NULL, 0, 0);
        var_init(l_exper->instru);
	  VAR_io_id for (i=0;i<INSTRU;i++)	if (var_param(i)) l_exper->prof_par++;
        }
     else  l_exper->onde=fget_var(2, 2,VAR_STD,T_LAMBDA,(void *) txt_onde, NULL, NULL, VAR_MODE_GT0, 0);
     
     if (l_exper->nature & EXPER_ENER_RESOL )
		   l_exper->c_exp=fget_var(1, 0, VAR_STD,"C_EXP",(void *) txt_c_exp, 
			    NULL, NULL, VAR_MODE_GT0 || VAR_MODE_ABS, 0.01);
        
	l_exper->n_var = n_var - l_exper->n_var;
	pop_err_msg();
 	switch (l_exper->nature & (EXPER_RX | EXPER_N | EXPER_RXA | EXPER_NM))
	{	case EXPER_RX  :  return (LECT_NAT_RX);
		case EXPER_N   :  return (LECT_NAT_N);
		case EXPER_RXA :  return (LECT_NAT_RXA | LECT_NAT_RX);
		case EXPER_NM  :  return (LECT_NAT_NM | LECT_NAT_N);
		}
   return 0;
	}

/* ------------------------------------------------------------------ */

